@@ -1,3 +1,4 @@
+from copy import deepcopy
 from typing import Tuple, Union
 import functools
 import torch
@@ -45,6 +46,10 @@ class CategoricalTensor(torch.Tensor):
         # return func(*args, **kwargs)
         raise NotImplementedError(func)
 
+class NoStackCatTensor(CategoricalTensor):
+    pass
+
+
 def implements(torch_function):
     """Register a torch function override for ScalarTensor"""
     @functools.wraps(torch_function)
@@ -53,6 +58,10 @@ def implements(torch_function):
         return func
     return decorator
 
+@implements(torch.Tensor.__deepcopy__)
+def ct_deepcopy(ct, memo):
+    return CategoricalTensor(deepcopy(ct.tensor, memo), deepcopy(ct.num_classes, memo))
+
 @implements(torch.split)
 def split(ct, *args, **kwargs):
     """ No error checking for now"""
@@ -67,6 +76,10 @@ def cat_tensor_len(ct):
 def cuda(ct):
     return CategoricalTensor(ct.tensor.cuda(), num_classes=ct.num_classes)
 
+@implements(torch.Tensor.cpu)
+def cpu(ct):
+    return CategoricalTensor(ct.tensor.cpu(), num_classes=ct.num_classes)
+
 @implements(torch.Tensor.to)
 def to(ct, device):
     return CategoricalTensor(ct.tensor.to(device), num_classes=ct.num_classes)
